pub struct VerifierData { /* private fields */ }Expand description
Collection of structs/objects that the Verifier will use in order to
de/serialize data needed for Circuit proof verification.
This structure can be seen as a link between the Circuit public input
positions and the VerifierKey that the Verifier needs to use.
Implementations
Sourceimpl VerifierData
impl VerifierData
Sourcepub const fn new(key: VerifierKey, public_inputs_indexes: Vec<u32>) -> Self
pub const fn new(key: VerifierKey, public_inputs_indexes: Vec<u32>) -> Self
Creates a new VerifierData from a VerifierKey and the public
input positions of the circuit that it represents.
Sourcepub const fn key(&self) -> &VerifierKey
pub const fn key(&self) -> &VerifierKey
Returns a reference to the contained VerifierKey.
Sourcepub fn public_inputs_indexes(&self) -> &[u32]
pub fn public_inputs_indexes(&self) -> &[u32]
Returns a reference to the contained Public Input positions.
Sourcepub fn to_var_bytes(&self) -> Vec<u8> ⓘ
pub fn to_var_bytes(&self) -> Vec<u8> ⓘ
Deserializes the VerifierData into a vector of bytes.
Sourcepub fn from_slice(buf: &[u8]) -> Result<Self, Error>
pub fn from_slice(buf: &[u8]) -> Result<Self, Error>
Serializes VerifierData from a slice of bytes.