pub struct Constraint { /* private fields */ }Expand description
Constraint representation containing the coefficients of a polynomial evaluation
Implementations
Sourceimpl Constraint
impl Constraint
Sourcepub const fn new() -> Self
pub const fn new() -> Self
Initiate the composition of a new selector description of a circuit.
Sourcepub fn mult<T: Into<BlsScalar>>(self, s: T) -> Self
pub fn mult<T: Into<BlsScalar>>(self, s: T) -> Self
Set s as the polynomial selector for the multiplication coefficient.
Sourcepub fn left<T: Into<BlsScalar>>(self, s: T) -> Self
pub fn left<T: Into<BlsScalar>>(self, s: T) -> Self
Set s as the polynomial selector for the left coefficient.
Sourcepub fn right<T: Into<BlsScalar>>(self, s: T) -> Self
pub fn right<T: Into<BlsScalar>>(self, s: T) -> Self
Set s as the polynomial selector for the right coefficient.
Sourcepub fn output<T: Into<BlsScalar>>(self, s: T) -> Self
pub fn output<T: Into<BlsScalar>>(self, s: T) -> Self
Set s as the polynomial selector for the output coefficient.
Sourcepub fn fourth<T: Into<BlsScalar>>(self, s: T) -> Self
pub fn fourth<T: Into<BlsScalar>>(self, s: T) -> Self
Set s as the polynomial selector for the fourth (advice) coefficient.
Sourcepub fn constant<T: Into<BlsScalar>>(self, s: T) -> Self
pub fn constant<T: Into<BlsScalar>>(self, s: T) -> Self
Set s as the polynomial selector for the constant of the constraint.