pub struct PreprocessedLookupTable {
pub n: u32,
/* private fields */
}Expand description
This table will be the preprocessed version of the precomputed table, T, with arity 4. This structure is passed to the proof alongside the table of witness values.
Fields
n: u32This is the circuit size
Implementations
Sourceimpl PreprocessedLookupTable
impl PreprocessedLookupTable
Sourcepub fn preprocess(
table: &LookupTable,
commit_key: &CommitKey,
n: u32,
) -> Result<Self, Error>
pub fn preprocess( table: &LookupTable, commit_key: &CommitKey, n: u32, ) -> Result<Self, Error>
This function takes in a precomputed look up table and pads it to the length of the circuit entries, as a power of 2. The function then interpolates a polynomial from the padded table and makes a commitment to the poly. The outputted struct will be used in the proof alongside our circuit witness table.