WitnessTable

Struct WitnessTable 

pub struct WitnessTable {
    pub f_1: MultiSet,
    pub f_2: MultiSet,
    pub f_3: MultiSet,
    pub f_4: MultiSet,
}
This witness table contains quieries to a lookup table for lookup gates This table is of arity 3.

Fields

f_1: MultiSet

This column represents the first values inside the lookup table. At gate checks, this can be regarded as the first wire

f_2: MultiSet

This column represents the first values inside the lookup table. At gate checks, this can be regarded as the second wire

f_3: MultiSet

This column represents the first values inside the lookup table. At gate checks, this can be regarded as the third wire

f_4: MultiSet

This column represents the first values inside the lookup table. At gate checks, this can be regarded as the fourth wire

Implementations

Source

impl WitnessTable

pub fn new() -> Self

Initialses empty witness table of arity 4

pub fn from_wire_values( &mut self, left_wire_val: BlsScalar, right_wire_val: BlsScalar, output_wire_val: BlsScalar, fourth_wire_val: BlsScalar, )

This allows the witness table to be filled directly without taking any vaules, or the the results, from the lookup table. If the values do no exists in the lookup table, then the proof will fail when witness and preprocessed tables are concatenated.

pub fn value_from_table( &mut self, lookup_table: &LookupTable, left_wire_val: BlsScalar, right_wire_val: BlsScalar, fourth_wire_val: BlsScalar, ) -> Result<(), Error>

Attempts to look up a value from a lookup table. If successful, all four elements are pushed to their respective multisets.