Proof

Struct Proof 

pub struct Proof { /* private fields */ }
A Proof is a composition of Commitments to the Witness, Permutation, Quotient, Shifted and Opening polynomials as well as the ProofEvaluations.

It’s main goal is to allow the Verifier to formally verify that the secret witnesses used to generate the Proof satisfy a circuit that both Prover and Verifier have in common succintly and without any capabilities of adquiring any kind of knowledge about the witness used to construct the Proof.