Verifier

Struct Verifier 

pub struct Verifier {
    pub verifier_key: Option<VerifierKey>,
    pub preprocessed_transcript: Transcript,
    /* private fields */
}
Abstraction structure designed verify Proofs.

Fields

verifier_key: Option<VerifierKey>

VerificationKey which is used to verify a specific PLONK circuit

preprocessed_transcript: Transcript

Store the messages exchanged during the preprocessing stage This is copied each time, we make a proof, so that we can use the same verifier to Verify multiple proofs from the same circuit. If this is not copied, then the verification procedure will modify the transcript, making it unusable for future proofs.

Implementations

Source

impl Verifier

pub fn new(label: &'static [u8]) -> Verifier

Creates a new Verifier instance.

pub fn with_size(label: &'static [u8], size: usize) -> Verifier

Creates a new Verifier instance with some expected size.

pub const fn gates(&self) -> u32

Returns the number of gates in the circuit.

pub fn composer_mut(&mut self) -> &mut TurboComposer

Mutable borrow of the TurboComposer.

pub fn preprocess(&mut self, commit_key: &CommitKey) -> Result<(), Error>

Preprocess a circuit to obtain a VerifierKey and a circuit descriptor so that the Verifier instance can verify Proofs for this circuit descriptor instance.

pub fn key_transcript(&mut self, label: &'static [u8], message: &[u8])

Keys the Transcript with additional seed information Wrapper around Transcript::append_message.

pub fn verify( &self, proof: &Proof, opening_key: &OpeningKey, public_inputs: &[BlsScalar], ) -> Result<(), Error>

Verifies a Proof.