pub struct VerifierKey { /* private fields */ }Expand description
PLONK circuit Verification Key.
This structure is used by the Verifier in order to verify a
Proof.
Implementations
Sourceimpl VerifierKey
impl VerifierKey
Sourcepub const fn padded_gates(&self) -> u32
pub const fn padded_gates(&self) -> u32
Returns the Circuit size padded to the next power of two.