Constraint

Struct Constraint 

pub struct Constraint { /* private fields */ }
Constraint representation containing the coefficients of a polynomial evaluation

Implementations

Source

impl Constraint

pub const fn new() -> Self

Initiate the composition of a new selector description of a circuit.

pub fn mult<T: Into<BlsScalar>>(self, s: T) -> Self

Set s as the polynomial selector for the multiplication coefficient.

pub fn left<T: Into<BlsScalar>>(self, s: T) -> Self

Set s as the polynomial selector for the left coefficient.

pub fn right<T: Into<BlsScalar>>(self, s: T) -> Self

Set s as the polynomial selector for the right coefficient.

pub fn output<T: Into<BlsScalar>>(self, s: T) -> Self

Set s as the polynomial selector for the output coefficient.

pub fn fourth<T: Into<BlsScalar>>(self, s: T) -> Self

Set s as the polynomial selector for the fourth (advice) coefficient.

pub fn constant<T: Into<BlsScalar>>(self, s: T) -> Self

Set s as the polynomial selector for the constant of the constraint.

pub fn public<T: Into<BlsScalar>>(self, s: T) -> Self

Set s as the public input of the constraint evaluation.

pub fn a(self, w: Witness) -> Self

Set witness a wired to qM and qL

pub fn b(self, w: Witness) -> Self

Set witness b wired to qM and qR

pub fn o(self, w: Witness) -> Self

Set witness o wired to qO

pub fn d(self, w: Witness) -> Self

Set witness d wired to the fourth/advice q4 coefficient