This module is a self contained file which explains how PLONK and its protocol components work in our library.
Table of Contents
The following section explains the workings of PLONK components:
This section explains how elements of the aggregated PLONK protocol work and some of the unique features in this library:
Modules
- commitment_
schemes - This module explains the inner workings of commitment schemes.
- kzg10_
docs - In this module we show how, and why, the KZG10 polynomial commitment scheme has been implemented for this PLONK implementation.
- prove_
verify - This module contains the notes on how the prover algorithm is constructed for PLONK.
- snark_
construction - This module contains the methodology of how zk-SNARKS are constructed.