Proving system
Structs
- Proof
- A Proof is a composition of
Commitments to the Witness, Permutation, Quotient, Shifted and Opening polynomials as well as theProofEvaluations. - Prover
- Abstraction structure designed to construct a circuit and generate
Proofs for it. - Prover
Key - PLONK circuit Proving Key.
- Verifier
- Abstraction structure designed verify
Proofs. - Verifier
Key - PLONK circuit Verification Key.